/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GP/src/InitHalfConstrainedOp.cpp
 *  \brief  Source code of class GP::InitHalfConstrainedOp.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.6 $
 *  $Date: 2004/07/04 21:45:31 $
 */

#include "beagle/GP.hpp"

#include <sstream>

using namespace Beagle;


/*!
 *  \brief Construct a constrained GP tree "half-and-half" initialization operator.
 *  \param inReproProbaName Reproduction probability parameter name used in register. 
 *  \param inName Name of the operator.
 */
GP::InitHalfConstrainedOp::InitHalfConstrainedOp(std::string inReproProbaName,
                                                 std::string inName) :
  Beagle::InitializationOp(inReproProbaName, inName),
  mInitFullOp(inReproProbaName),
  mInitGrowOp(inReproProbaName)
{ }


/*!
 *  \brief Initialize the constrained GP tree "half-and-half" intialization operator.
 *  \param ioSystem System of the evolution.
 */
void GP::InitHalfConstrainedOp::initialize(Beagle::System& ioSystem)
{
  Beagle::InitializationOp::initialize(ioSystem);
  mInitFullOp.initialize(ioSystem);
  mInitGrowOp.initialize(ioSystem);
}


/*!
 *  \brief Initialize the constrained trees of an individual using the "half-and-half" approach.
 *  \param outIndividual Individual to initialize.
 *  \param ioContext Evolution context.
 */
void GP::InitHalfConstrainedOp::initIndividual(Beagle::Individual& outIndividual,
                                               Beagle::Context& ioContext)
{
  if(ioContext.getSystem().getRandomizer().rollUniform(0.0, 1.0) < 0.5)
    mInitFullOp.initIndividual(outIndividual, ioContext);
  else mInitGrowOp.initIndividual(outIndividual, ioContext);
}

