/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   beagle/GP/src/PrimitiveSuperSet.cpp
 *  \brief  Implementation of the type GP::PrimitiveSuperSet.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.4 $
 *  $Date: 2004/04/25 23:12:20 $
 */

#include "beagle/GP.hpp"

using namespace Beagle;


/*!
 *  \brief Initialize the super set of primitive.
 */
void GP::PrimitiveSuperSet::initialize(GP::System& ioSystem)
{
  for(unsigned int i=0; i<size(); i++) (*this)[i]->initialize(ioSystem);
}


/*!
 *  \brief Insert a nes set of primitives in the super set.
 *  \param inPrimitiveSet Inserted primitive set.
 */
void GP::PrimitiveSuperSet::insert(GP::PrimitiveSet::Handle inPrimitiveSet)
{
  push_back(inPrimitiveSet);
}

/*!
 *  \brief Read a primitive super set from a Beagle XML stream.
 *  \param inNode XML node used to read the super set from.
 *  \throw Beagle::IOException If size atribute not present.
 */
void GP::PrimitiveSuperSet::read(XMLNode::Handle& inNode)
{
  if((inNode->getType() != XMLNode::eTag) || (inNode->getTagName() != "PrimitiveSuperSet"))
    throw Beagle_IOExceptionNodeM((*inNode), "tag <PrimitiveSuperSet> expected!");
  size_type lPSIndex = 0;
  for(XMLNode::Handle lChild=inNode->getFirstChild();
      lChild != NULL; lChild=lChild->getNextSibling()) {
    if((lChild->getType()==XMLNode::eTag) && (lChild->getTagName()=="PrimitiveSet")) {
      (*this)[lPSIndex++]->read(lChild);
    }
  }
}


/*!
 *  \brief Write a primitive super set into a Beagle XML stream.
 *  \param ioStreamer XMl streamer to write the super set into.
 */
void GP::PrimitiveSuperSet::write(XMLStreamer& ioStreamer) const
{
  ioStreamer.openTag("PrimitiveSuperSet");
  for(size_type i=0; i<size(); i++) (*this)[i]->write(ioStreamer);
  ioStreamer.closeTag();
}
