/*
 *  Open BEAGLE
 *  Copyright (C) 2001-2004 by Christian Gagne and Marc Parizeau
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  Contact:
 *  Laboratoire de Vision et Systemes Numeriques
 *  Departement de genie electrique et de genie informatique
 *  Universite Laval, Quebec, Canada, G1K 7P4
 *  http://vision.gel.ulaval.ca
 *
 */

/*!
 *  \file   EqualTo.hpp
 *  \brief  Definition of EqualTo primitive used in the spambase problem.
 *  \author Christian Gagne
 *  \author Marc Parizeau
 *  $Revision: 1.4 $
 *  $Date: 2004/04/25 23:12:23 $
 */

#ifndef EqualTo_hpp
#define EqualTo_hpp

#include <string>
#include "beagle/GP.hpp"


/*!
 *  \class EqualTo EqualTo.hpp "EqualTo.hpp"
 *  \brief EqualTo primitive that compare equality of two Double and return a Bool.
 *  \ingroup Spambase
 */
class EqualTo : public Beagle::GP::Primitive {

public:

  //! EqualTo allocator type.
  typedef Beagle::AllocatorT<EqualTo,Beagle::GP::Primitive::Alloc>
          Alloc;
  //!< EqualTo handle type.
  typedef Beagle::PointerT<EqualTo,Beagle::GP::Primitive::Handle>
          Handle;
  //!< EqualTo bag type.
  typedef Beagle::ContainerT<EqualTo,Beagle::GP::Primitive::Bag>
          Bag;

  explicit EqualTo(std::string inName="EQ");
  virtual ~EqualTo() { }

  virtual std::string getArgType(unsigned int inN) const;
  virtual std::string getReturnType() const;
  virtual void        execute(Beagle::GP::Datum& outDatum, Beagle::GP::Context& ioContext);

};


#endif // EqualTo_hpp
